@@ -11,23 +11,14 @@ extern "C" {
 #include "loader_win_utils.c"
 #endif
 
-/* Define ptls getter, as this cannot be defined within a shared library. */
-#if !defined(_OS_WINDOWS_) && !defined(_OS_DARWIN_)
-JL_DLLEXPORT JL_CONST_FUNC void * jl_get_ptls_states_static(void)
-{
-    /* Because we can't #include <julia.h> in this file, we define a TLS state object with
-     * hopefully enough room; at last check, the `jl_tls_states_t` struct was <16KB. */
-    static __attribute__((tls_model("local-exec"))) __thread char tls_states[32768];
-    return &tls_states;
-}
-#endif
+JULIA_DEFINE_FAST_TLS
 
 #ifdef _OS_WINDOWS_
 int mainCRTStartup(void)
 {
     int argc;
     LPWSTR * wargv = CommandLineToArgv(GetCommandLine(), &argc);
-    char ** argv = (char **)malloc(sizeof(char *)*(argc+ 1));
+    char ** argv = (char **)malloc(sizeof(char*) * (argc + 1));
     setup_stdio();
 #else
 int main(int argc, char * argv[])
@@ -36,7 +27,7 @@ int main(int argc, char * argv[])
 
     // Convert Windows wchar_t values to UTF8
 #ifdef _OS_WINDOWS_
-    for (int i=0; i<argc; i++) {
+    for (int i = 0; i < argc; i++) {
         size_t max_arg_len = 4*wcslen(wargv[i]);
         argv[i] = (char *)malloc(max_arg_len);
         if (!wchar_to_utf8(wargv[i], argv[i], max_arg_len)) {
