@@ -131,6 +131,8 @@ static void create_PRUNTIME_FUNCTION(uint8_t *Code, size_t Size, StringRef fnnam
     tbl->BeginAddress = (DWORD)(Code - Section);
     tbl->EndAddress = (DWORD)(Code - Section + Size);
     tbl->UnwindData = (DWORD)(UnwindData - Section);
+    assert(Code >= Section && Code + Size <= Section + Allocated);
+    assert(UnwindData >= Section && UnwindData <= Section + Allocated);
 #else // defined(_CPU_X86_64_)
     Section += (uintptr_t)Code;
     mod_size = Size;
@@ -306,20 +308,13 @@ public:
         uint8_t *catchjmp = NULL;
         for (const object::SymbolRef &sym_iter : debugObj.symbols()) {
             StringRef sName = cantFail(sym_iter.getName());
-            uint8_t **pAddr = NULL;
-            if (sName.equals("__UnwindData")) {
-                pAddr = &UnwindData;
-            }
-            else if (sName.equals("__catchjmp")) {
-                pAddr = &catchjmp;
-            }
-            if (pAddr) {
+            if (sName.equals("__UnwindData") || sName.equals("__catchjmp")) {
                 uint64_t Addr = cantFail(sym_iter.getAddress());
                 auto Section = cantFail(sym_iter.getSection());
                 assert(Section != EndSection && Section->isText());
                 uint64_t SectionAddr = Section->getAddress();
-                sName = cantFail(Section->getName());
-                uint64_t SectionLoadAddr = getLoadAddress(sName);
+                StringRef secName = cantFail(Section->getName());
+                uint64_t SectionLoadAddr = getLoadAddress(secName);
                 assert(SectionLoadAddr);
                 if (SectionAddrCheck) // assert that all of the Sections are at the same location
                     assert(SectionAddrCheck == SectionAddr &&
@@ -331,7 +326,12 @@ public:
                     SectionWriteCheck = (uintptr_t)lookupWriteAddressFor(memmgr,
                             (void*)SectionLoadAddr);
                 Addr += SectionWriteCheck - SectionLoadAddr;
-                *pAddr = (uint8_t*)Addr;
+                if (sName.equals("__UnwindData")) {
+                    UnwindData = (uint8_t*)Addr;
+                }
+                else if (sName.equals("__catchjmp")) {
+                    catchjmp = (uint8_t*)Addr;
+                }
             }
         }
         assert(catchjmp);
@@ -354,6 +354,7 @@ public:
         UnwindData[6] = 1;    // first instruction
         UnwindData[7] = 0x50; // push RBP
         *(DWORD*)&UnwindData[8] = (DWORD)(catchjmp - (uint8_t*)SectionWriteCheck); // relative location of catchjmp
+        UnwindData -= SectionWriteCheck - SectionLoadCheck;
 #endif // defined(_OS_X86_64_)
 #endif // defined(_OS_WINDOWS_)
 
@@ -1077,6 +1078,14 @@ bool jl_dylib_DI_for_fptr(size_t pointer, object::SectionRef *Section, int64_t *
     struct link_map *extra_info;
     dladdr_success = dladdr1((void*)pointer, &dlinfo, (void**)&extra_info, RTLD_DL_LINKMAP) != 0;
 #else
+#ifdef _OS_DARWIN_
+    // On macOS 12, dladdr(-1, …) succeeds and returns the main executable image,
+    // despite there never actually being an image there. This is not what we want,
+    // as we use -1 as a known-invalid value e.g. in the test suite.
+    if (pointer == ~(size_t)0) {
+        return false;
+    }
+#endif
     dladdr_success = dladdr((void*)pointer, &dlinfo) != 0;
 #endif
     if (!dladdr_success || !dlinfo.dli_fname)
@@ -1123,6 +1132,7 @@ static int jl_getDylibFunctionInfo(jl_frame_t **frames, size_t pointer, int skip
     static IMAGEHLP_LINE64 frame_info_line;
     DWORD dwDisplacement = 0;
     JL_LOCK_NOGC(&jl_in_stackwalk);
+    jl_refresh_dbg_module_list();
     DWORD64 dwAddress = pointer;
     frame_info_line.SizeOfStruct = sizeof(IMAGEHLP_LINE64);
     if (SymGetLineFromAddr64(GetCurrentProcess(), dwAddress, &dwDisplacement, &frame_info_line)) {
