@@ -271,9 +271,7 @@
   (define (other x) (resolve-expansion-vars-with-new-env x env m parent-scope inarg))
   (case (car e)
     ((where) `(where ,(recur (cadr e)) ,@(map other (cddr e))))
-    ((|::|)  (if (length= e 2)
-                 `(|::| ,(other (cadr e)))
-                 `(|::| ,(recur (cadr e)) ,(other (caddr e)))))
+    ((|::|)  `(|::| ,(recur (cadr e)) ,(other (caddr e))))
     ((call)  `(call ,(other (cadr e))
                     ,@(map (lambda (x)
                              (resolve-expansion-vars-with-new-env x env m parent-scope #t))
@@ -457,14 +455,16 @@
 
 ;; decl-var that also identifies f in f()=...
 (define (decl-var* e)
-  (cond ((not (pair? e))       e)
-        ((eq? (car e) 'escape) '())
-        ((eq? (car e) 'call)   (decl-var* (cadr e)))
-        ((eq? (car e) '=)      (decl-var* (cadr e)))
-        ((eq? (car e) 'curly)  (decl-var* (cadr e)))
-        ((eq? (car e) '|::|)   (decl-var* (cadr e)))
-        ((eq? (car e) 'where)  (decl-var* (cadr e)))
-        (else                  (decl-var e))))
+  (if (pair? e)
+      (case (car e)
+        ((escape) '())
+        ((call)   (decl-var* (cadr e)))
+        ((=)      (decl-var* (cadr e)))
+        ((curly)  (decl-var* (cadr e)))
+        ((|::|)   (if (length= e 2) '() (decl-var* (cadr e))))
+        ((where)  (decl-var* (cadr e)))
+        (else     (decl-var e)))
+      e))
 
 (define (decl-vars* e)
   (if (and (pair? e) (eq? (car e) 'tuple))
