@@ -66,11 +66,12 @@ void jl_module_run_initializer(jl_module_t *m)
     jl_function_t *f = jl_module_get_initializer(m);
     if (f == NULL)
         return;
-    size_t last_age = jl_get_ptls_states()->world_age;
+    jl_task_t *ct = jl_current_task;
+    size_t last_age = ct->world_age;
     JL_TRY {
-        jl_get_ptls_states()->world_age = jl_world_counter;
+        ct->world_age = jl_world_counter;
         jl_apply(&f, 1);
-        jl_get_ptls_states()->world_age = last_age;
+        ct->world_age = last_age;
     }
     JL_CATCH {
         if (jl_initerror_type == NULL) {
@@ -115,7 +116,7 @@ static int jl_is__toplevel__mod(jl_module_t *mod)
 // TODO: add locks around global state mutation operations
 static jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex)
 {
-    jl_ptls_t ptls = jl_get_ptls_states();
+    jl_task_t *ct = jl_current_task;
     assert(ex->head == module_sym);
     if (jl_array_len(ex->args) != 3 || !jl_is_expr(jl_exprarg(ex, 2))) {
         jl_error("syntax: malformed module expression");
@@ -148,8 +149,8 @@ static jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex
         newm->parent = parent_module;
         jl_binding_t *b = jl_get_binding_wr(parent_module, name, 1);
         jl_declare_constant(b);
-        jl_value_t *old = jl_atomic_compare_exchange(&b->value, NULL, (jl_value_t*)newm);
-        if (old != NULL) {
+        jl_value_t *old = NULL;
+        if (!jl_atomic_cmpswap(&b->value, &old, (jl_value_t*)newm)) {
             if (!jl_is_module(old)) {
                 jl_errorf("invalid redefinition of constant %s", jl_symbol_name(name));
             }
@@ -173,7 +174,7 @@ static jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex
         jl_base_module = newm;
     }
 
-    size_t last_age = ptls->world_age;
+    size_t last_age = ct->world_age;
 
     // add standard imports unless baremodule
     if (std_imports) {
@@ -189,13 +190,13 @@ static jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex
     jl_array_t *exprs = ((jl_expr_t*)jl_exprarg(ex, 2))->args;
     for (int i = 0; i < jl_array_len(exprs); i++) {
         // process toplevel form
-        ptls->world_age = jl_world_counter;
+        ct->world_age = jl_world_counter;
         form = jl_expand_stmt_with_loc(jl_array_ptr_ref(exprs, i), newm, jl_filename, jl_lineno);
-        ptls->world_age = jl_world_counter;
+        ct->world_age = jl_world_counter;
         (void)jl_toplevel_eval_flex(newm, form, 1, 1);
     }
     newm->primary_world = jl_world_counter;
-    ptls->world_age = last_age;
+    ct->world_age = last_age;
 
 #if 0
     // some optional post-processing steps
@@ -267,7 +268,7 @@ static jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex
 
 static jl_value_t *jl_eval_dot_expr(jl_module_t *m, jl_value_t *x, jl_value_t *f, int fast)
 {
-    jl_ptls_t ptls = jl_get_ptls_states();
+    jl_task_t *ct = jl_current_task;
     jl_value_t **args;
     JL_GC_PUSHARGS(args, 3);
     args[1] = jl_toplevel_eval_flex(m, x, fast, 0);
@@ -278,10 +279,10 @@ static jl_value_t *jl_eval_dot_expr(jl_module_t *m, jl_value_t *x, jl_value_t *f
     }
     else {
         args[0] = jl_eval_global_var(jl_base_relative_to(m), jl_symbol("getproperty"));
-        size_t last_age = ptls->world_age;
-        ptls->world_age = jl_world_counter;
+        size_t last_age = ct->world_age;
+        ct->world_age = jl_world_counter;
         args[0] = jl_apply(args, 3);
-        ptls->world_age = last_age;
+        ct->world_age = last_age;
     }
     JL_GC_POP();
     return args[0];
@@ -303,7 +304,7 @@ JL_DLLEXPORT jl_module_t *jl_base_relative_to(jl_module_t *m)
     return jl_top_module;
 }
 
-static void expr_attributes(jl_value_t *v, int *has_intrinsics, int *has_defs)
+static void expr_attributes(jl_value_t *v, int *has_intrinsics, int *has_defs, int *has_opaque)
 {
     if (!jl_is_expr(v))
         return;
@@ -334,6 +335,10 @@ static void expr_attributes(jl_value_t *v, int *has_intrinsics, int *has_defs)
         *has_intrinsics = 1;
         return;
     }
+    else if (head == new_opaque_closure_sym) {
+        *has_opaque = 1;
+        return;
+    }
     else if (head == call_sym && jl_expr_nargs(e) > 0) {
         jl_value_t *called = NULL;
         jl_value_t *f = jl_exprarg(e, 0);
@@ -363,7 +368,7 @@ static void expr_attributes(jl_value_t *v, int *has_intrinsics, int *has_defs)
     for (i = 0; i < jl_array_len(e->args); i++) {
         jl_value_t *a = jl_exprarg(e, i);
         if (jl_is_expr(a))
-            expr_attributes(a, has_intrinsics, has_defs);
+            expr_attributes(a, has_intrinsics, has_defs, has_opaque);
     }
 }
 
@@ -372,17 +377,17 @@ int jl_code_requires_compiler(jl_code_info_t *src)
     jl_array_t *body = src->code;
     assert(jl_typeis(body, jl_array_any_type));
     size_t i;
-    int has_intrinsics = 0, has_defs = 0;
+    int has_intrinsics = 0, has_defs = 0, has_opaque = 0;
     for(i=0; i < jl_array_len(body); i++) {
         jl_value_t *stmt = jl_array_ptr_ref(body,i);
-        expr_attributes(stmt, &has_intrinsics, &has_defs);
+        expr_attributes(stmt, &has_intrinsics, &has_defs, &has_opaque);
         if (has_intrinsics)
             return 1;
     }
     return 0;
 }
 
-static void body_attributes(jl_array_t *body, int *has_intrinsics, int *has_defs, int *has_loops)
+static void body_attributes(jl_array_t *body, int *has_intrinsics, int *has_defs, int *has_loops, int *has_opaque)
 {
     size_t i;
     *has_loops = 0;
@@ -398,7 +403,7 @@ static void body_attributes(jl_array_t *body, int *has_intrinsics, int *has_defs
                     *has_loops = 1;
             }
         }
-        expr_attributes(stmt, has_intrinsics, has_defs);
+        expr_attributes(stmt, has_intrinsics, has_defs, has_opaque);
     }
 }
 
@@ -407,19 +412,19 @@ static jl_module_t *call_require(jl_module_t *mod, jl_sym_t *var) JL_GLOBALLY_RO
     static jl_value_t *require_func = NULL;
     int build_mode = jl_generating_output();
     jl_module_t *m = NULL;
-    jl_ptls_t ptls = jl_get_ptls_states();
+    jl_task_t *ct = jl_current_task;
     if (require_func == NULL && jl_base_module != NULL) {
         require_func = jl_get_global(jl_base_module, jl_symbol("require"));
     }
     if (require_func != NULL) {
-        size_t last_age = ptls->world_age;
-        ptls->world_age = (build_mode ? jl_base_module->primary_world : jl_world_counter);
+        size_t last_age = ct->world_age;
+        ct->world_age = (build_mode ? jl_base_module->primary_world : jl_world_counter);
         jl_value_t *reqargs[3];
         reqargs[0] = require_func;
         reqargs[1] = (jl_value_t*)mod;
         reqargs[2] = (jl_value_t*)var;
         m = (jl_module_t*)jl_apply(reqargs, 3);
-        ptls->world_age = last_age;
+        ct->world_age = last_age;
     }
     if (m == NULL || !jl_is_module(m)) {
         jl_errorf("failed to load module %s", jl_symbol_name(var));
@@ -615,7 +620,7 @@ static void jl_eval_errorf(jl_module_t *m, const char* fmt, ...)
 
 jl_value_t *jl_toplevel_eval_flex(jl_module_t *JL_NONNULL m, jl_value_t *e, int fast, int expanded)
 {
-    jl_ptls_t ptls = jl_get_ptls_states();
+    jl_task_t *ct = jl_current_task;
     if (!jl_is_expr(e)) {
         if (jl_is_linenode(e)) {
             jl_lineno = jl_linenode_line(e);
@@ -648,7 +653,7 @@ jl_value_t *jl_toplevel_eval_flex(jl_module_t *JL_NONNULL m, jl_value_t *e, int
         }
     }
 
-    if (ptls->in_pure_callback) {
+    if (ct->ptls->in_pure_callback) {
         jl_error("eval cannot be used in a generated function");
     }
 
@@ -656,11 +661,11 @@ jl_value_t *jl_toplevel_eval_flex(jl_module_t *JL_NONNULL m, jl_value_t *e, int
     jl_code_info_t *thk = NULL;
     JL_GC_PUSH3(&mfunc, &thk, &ex);
 
-    size_t last_age = ptls->world_age;
+    size_t last_age = ct->world_age;
     if (!expanded && jl_needs_lowering(e)) {
-        ptls->world_age = jl_world_counter;
+        ct->world_age = jl_world_counter;
         ex = (jl_expr_t*)jl_expand_with_loc_warn(e, m, jl_filename, jl_lineno);
-        ptls->world_age = last_age;
+        ct->world_age = last_age;
     }
     jl_sym_t *head = jl_is_expr(ex) ? ex->head : NULL;
 
@@ -843,12 +848,12 @@ jl_value_t *jl_toplevel_eval_flex(jl_module_t *JL_NONNULL m, jl_value_t *e, int
         return (jl_value_t*)ex;
     }
 
-    int has_intrinsics = 0, has_defs = 0, has_loops = 0;
+    int has_intrinsics = 0, has_defs = 0, has_loops = 0, has_opaque = 0;
     assert(head == thunk_sym);
     thk = (jl_code_info_t*)jl_exprarg(ex, 0);
     assert(jl_is_code_info(thk));
     assert(jl_typeis(thk->code, jl_array_any_type));
-    body_attributes((jl_array_t*)thk->code, &has_intrinsics, &has_defs, &has_loops);
+    body_attributes((jl_array_t*)thk->code, &has_intrinsics, &has_defs, &has_loops, &has_opaque);
 
     jl_value_t *result;
     if (has_intrinsics || (!has_defs && fast && has_loops &&
@@ -864,16 +869,19 @@ jl_value_t *jl_toplevel_eval_flex(jl_module_t *JL_NONNULL m, jl_value_t *e, int
         // TODO: This is still not correct since an `eval` can happen elsewhere, but it
         // helps in common cases.
         size_t world = jl_world_counter;
-        ptls->world_age = world;
+        ct->world_age = world;
         if (!has_defs && jl_get_module_infer(m) != 0) {
             (void)jl_type_infer(mfunc, world, 0);
         }
         result = jl_invoke(/*func*/NULL, /*args*/NULL, /*nargs*/0, mfunc);
-        ptls->world_age = last_age;
+        ct->world_age = last_age;
     }
     else {
         // use interpreter
         assert(thk);
+        if (has_opaque) {
+            jl_resolve_globals_in_ir((jl_array_t*)thk->code, m, NULL, 0);
+        }
         result = jl_interpret_toplevel_thunk(m, thk);
     }
 
@@ -914,10 +922,17 @@ static void jl_check_open_for(jl_module_t *m, const char* funcname)
     }
 }
 
+JL_DLLEXPORT void jl_check_top_level_effect(jl_module_t *m, char *fname)
+{
+    if (jl_current_task->ptls->in_pure_callback)
+        jl_errorf("%s cannot be used in a generated function", fname);
+    jl_check_open_for(m, fname);
+}
+
 JL_DLLEXPORT jl_value_t *jl_toplevel_eval_in(jl_module_t *m, jl_value_t *ex)
 {
-    jl_ptls_t ptls = jl_get_ptls_states();
-    if (ptls->in_pure_callback)
+    jl_task_t *ct = jl_current_task;
+    if (ct->ptls->in_pure_callback)
         jl_error("eval cannot be used in a generated function");
     jl_check_open_for(m, "eval");
     jl_value_t *v = NULL;
@@ -944,7 +959,8 @@ JL_DLLEXPORT jl_value_t *jl_infer_thunk(jl_code_info_t *thk, jl_module_t *m)
     jl_method_instance_t *li = method_instance_for_thunk(thk, m);
     JL_GC_PUSH1(&li);
     jl_resolve_globals_in_ir((jl_array_t*)thk->code, m, NULL, 0);
-    jl_code_info_t *src = jl_type_infer(li, jl_get_ptls_states()->world_age, 0);
+    jl_task_t *ct = jl_current_task;
+    jl_code_info_t *src = jl_type_infer(li, ct->world_age, 0);
     JL_GC_POP();
     if (src)
         return src->rettype;
@@ -964,8 +980,8 @@ static jl_value_t *jl_parse_eval_all(jl_module_t *module, jl_value_t *text,
     if (!jl_is_string(text) || !jl_is_string(filename)) {
         jl_errorf("Expected `String`s for `text` and `filename`");
     }
-    jl_ptls_t ptls = jl_get_ptls_states();
-    if (ptls->in_pure_callback)
+    jl_task_t *ct = jl_current_task;
+    if (ct->ptls->in_pure_callback)
         jl_error("cannot use include inside a generated function");
     jl_check_open_for(module, "include");
 
@@ -982,7 +998,7 @@ static jl_value_t *jl_parse_eval_all(jl_module_t *module, jl_value_t *text,
 
     int last_lineno = jl_lineno;
     const char *last_filename = jl_filename;
-    size_t last_age = jl_get_ptls_states()->world_age;
+    size_t last_age = ct->world_age;
     int lineno = 0;
     jl_lineno = 0;
     jl_filename = jl_string_data(filename);
@@ -999,7 +1015,7 @@ static jl_value_t *jl_parse_eval_all(jl_module_t *module, jl_value_t *text,
             }
             expression = jl_expand_with_loc_warn(expression, module,
                                                  jl_string_data(filename), lineno);
-            jl_get_ptls_states()->world_age = jl_world_counter;
+            ct->world_age = jl_world_counter;
             result = jl_toplevel_eval_flex(module, expression, 1, 1);
         }
     }
@@ -1009,7 +1025,7 @@ static jl_value_t *jl_parse_eval_all(jl_module_t *module, jl_value_t *text,
         goto finally; // skip jl_restore_excstack
     }
 finally:
-    jl_get_ptls_states()->world_age = last_age;
+    ct->world_age = last_age;
     jl_lineno = last_lineno;
     jl_filename = last_filename;
     if (err) {
