@@ -10,14 +10,14 @@ const PATH_list = String[]
 const LIBPATH_list = String[]
 
 # These get calculated in __init__()
-PATH = Ref("")
-LIBPATH = Ref("")
+const PATH = Ref("")
+const LIBPATH = Ref("")
 artifact_dir = ""
 cacert = ""
 
 function __init__()
-	global artifact_dir = dirname(Sys.BINDIR)
-	global cacert = normpath(Sys.BINDIR::String, Base.DATAROOTDIR, "julia", "cert.pem")
+    global artifact_dir = dirname(Sys.BINDIR)
+    global cacert = normpath(Sys.BINDIR::String, Base.DATAROOTDIR, "julia", "cert.pem")
 end
 
 # JLLWrappers API compatibility shims.  Note that not all of these will really make sense.
